type reg = int list

let ref_count = ref 32 
let new_reg _ = ref_count := !ref_count + 1; [!ref_count]

let sp = [2]

let temp0 = [30]
let temp1 = [31]

let nb_regs_ref = ref 2

let nb_regs () = !nb_regs_ref

let set_nb_regs i = nb_regs_ref := i


let rec to_string r = match r with
  | [0] -> "zero"
  | [1]  -> "ra"
  | [2]  -> "sp"
  | [3]  -> "gp"
  | [4]  -> "tp"
  | [5]  -> "t0"
  | [6]  -> "t1"
  | [7]  -> "t2"
  | [8]  -> "s0"
  | [9]  -> "s1"
  | [10] -> "a0"
  | [11] -> "a1"
  | [12] -> "a2"
  | [13] -> "a3"
  | [14] -> "a4"
  | [15] -> "a5"
  | [16] -> "a6"
  | [17] -> "a7"
  | [18] -> "s2"
  | [19] -> "s3"
  | [20] -> "s4"
  | [21] -> "s5"
  | [22] -> "s6"
  | [23] -> "s7"
  | [24] -> "s8"
  | [25] -> "s9"
  | [26] -> "s10"
  | [27] -> "s11"
  | [28] -> "t3"
  | [29] -> "t4"
  | [30] -> "t5"
  | [31] -> "t6"
  | [r] ->  if r < 32 then Printf.sprintf "x%i" r else
      Printf.sprintf "%%%i" (r - 32)
  | x::q -> to_string [x] ^ "|" ^ to_string q
  | [] -> failwith "empty register"

let rec to_dot r = match r with
  | [0] -> "zero"
  | [1]  -> "ra"
  | [2]  -> "sp"
  | [3]  -> "gp"
  | [4]  -> "tp"
  | [5]  -> "t0"
  | [6]  -> "t1"
  | [7]  -> "t2"
  | [8]  -> "s0"
  | [9]  -> "s1"
  | [10] -> "a0"
  | [11] -> "a1"
  | [12] -> "a2"
  | [13] -> "a3"
  | [14] -> "a4"
  | [15] -> "a5"
  | [16] -> "a6"
  | [17] -> "a7"
  | [18] -> "s2"
  | [19] -> "s3"
  | [20] -> "s4"
  | [21] -> "s5"
  | [22] -> "s6"
  | [23] -> "s7"
  | [24] -> "s8"
  | [25] -> "s9"
  | [26] -> "s10"
  | [27] -> "s11"
  | [28] -> "t3"
  | [29] -> "t4"
  | [30] -> "t5"
  | [31] -> "t6"
  | [r] ->  if r < 32 then Printf.sprintf "x%i" r else
      Printf.sprintf "pr%i" (r - 32)
  | x::q -> to_dot [x] ^ "|" ^ to_dot q
  | [] -> failwith "empty register"


        
let print_reg out r = Printf.fprintf out "%s" (to_string r) 

let assigned_sing v = v < 32

let is_real = List.exists assigned_sing

module Reg = struct
  type t = reg
let compare = compare
let equal = (=)
end

module RegSet = Set.Make(Reg)

type reg_set = RegSet.t

include RegSet

let real_set () =
  let rec aux accu k = 
    if k = 0 then accu
    else aux (add [k+1] accu) (k-1)
  in
  aux empty (nb_regs())
    

let from_int k =  [k + 4]

let from_string s = match s with
  | "zero" -> [0]
  | "ra" -> [1]
  | "sp" -> [2]
  | "gp" -> [3]
  | "tp" -> [4]
  | "t0" -> [5]
  | "t1" -> [6]
  | "t2" -> [7]
  | "s0" -> [8]
  | "s1" -> [9]
  | "a0" -> [10]
  | "a1" -> [11]
  | "a2" -> [12]
  | "a3" -> [13]
  | "a4" -> [14]
  | "a5" -> [15]
  | "a6" -> [16]
  | "a7" -> [17]
  | "s2" -> [18]
  | "s3" -> [19]
  | "s4" -> [20]
  | "s5" -> [21]
  | "s6" -> [22]
  | "s7" -> [23]
  | "s8" -> [24]
  | "s9" -> [25]
  | "s10" -> [26]
  | "s11" -> [27]
  | "t3" -> [28]
  | "t4" -> [29]
  | "t5" -> [30]
  | "t6" -> [31]
  | _ -> if s.[0] = 'x' then 
      [int_of_string (String.sub s 1 (String.length s - 1))]
    else if s.[0] = '%' then
      [int_of_string (String.sub s 1 (String.length s - 1))+32]
    else failwith "Not a register"


let join = (@)

let to_color c= 
  let aux = function
  |  5 -> 0xff0000
  |  6 -> 0x00ff00
  |  7 -> 0x0000ff
  |  8 -> 0x00ffff
  |  9 -> 0xff00ff
  | 10 -> 0xffff00
  | 11 -> 0x800000
  | 12 -> 0x008000
  | 13 -> 0x000080
  | 14 -> 0x008080
  | 15 -> 0x800080
  | 16 -> 0x808000
  | n -> (n - 16) * 0x002000 
  in match c with
      [r] -> aux r
    | _ -> failwith "to_color : not a normal register"

let rec real = function 
  | [] -> raise Not_found
  | x::q when is_real [x] -> [x]
  | x::q -> real q

let real_regs : unit -> reg_set =
  let rec aux accu k = 
    if k = 0 then accu
    else aux (add (from_int k) accu) (k-1)
  in
  fun () -> aux empty !nb_regs_ref


     
