open Register

type pseudo_register = reg

type arg = Cte of int | Reg of pseudo_register

type label = string              

type rtl_node = {
  label : label;
  op : string;
  dest : pseudo_register option;
  args : arg list;
  exits : label list;
}
              
type rtl = rtl_node list

type riscv = rtl_node

let pp_pseudo_register out i =
  print_reg out i

let pp_arg out = function
  | Reg r -> pp_pseudo_register out r
  | Cte k -> Printf.fprintf out "%i" k
    
let rec pp_args out = function
  | [] -> ()
  | [x] -> Printf.fprintf out "%a" pp_arg x;
  | x::q ->
     Printf.fprintf out "%a, " pp_arg x;
     pp_args out q

let rec pp_exits out = function
  | [] -> failwith "Node without exit labels"
  | [x] -> Printf.fprintf out "%s" x
  | x::q ->
     Printf.fprintf out "%s, " x;
     pp_exits out q
  

let pp_rtl_node out n =
  Printf.fprintf out "%s: %s %a -> %a\n"
    n.label
    n.op
    pp_args (match n.dest with
               Some r -> Reg r::n.args
             | None -> n.args)
    pp_exits n.exits
    
                       
let pp_rtl out p =
  List.iter (pp_rtl_node out) p


let print_riscv out n = 
    Printf.fprintf out "%s %a\n"
    n.op
    pp_args (match n.dest with
               Some r -> Reg r::n.args
             | None -> n.args)
