%{
  open Riscv
  open Register

  let fresh_label =
    let c = ref 0 in
    fun () -> incr c; "l" ^ string_of_int !c
%}
%token <int> INT
%token <string> REG INS
%token LPAR RPAR EOL LW SW ECALL COMMA EOF
%start s
%type <Riscv.rtl> s
%%
s: es EOF { $1 (fresh_label ()) }

es: e EOL es { fun l -> let i, ex = $1 l in i::$3 ex }
|  EOL es { $2 }
| { fun l -> [] }

e:
LW REG COMMA INT LPAR REG RPAR {
  fun l -> let ex = fresh_label() in
  { label = ex;
    op = l;
    dest = Some (from_string $2);
    args = [Reg (from_string $6); Cte $4];
    exits = [ex]
  }, ex
}
  | SW REG COMMA INT LPAR REG RPAR {
	 fun l -> let ex = fresh_label() in
		  { label = ex;
		    op = "sw";
		    dest = None;
		    args = [Reg (from_string $2); Reg (from_string $6); Cte $4];
		    exits = [ex]
		  }, ex
       }
  | INS REG COMMA INT {
	    fun l -> let ex = fresh_label() in
		     { label = ex;
		       op = $1;
	    dest = Some (from_string $2);
	    args = [Cte $4];
	    exits = [ex]
	  }, ex
	}
  | INS REG COMMA REG {
	  fun l -> let ex = fresh_label() in
		   { label = ex;
	    op = $1;
	    dest = Some (from_string $2);
	    args = [Reg (from_string $4)];
	    exits = [ex]
	   }, ex
	}
  | INS REG COMMA REG COMMA INT {
	  fun l -> let ex = fresh_label() in
		   { label = ex;
	    op = $1;
	    dest = Some (from_string $2);
	    args = [Reg (from_string $4); Cte $6];
	    exits = [ex]
	  }, ex
	}
  | INS REG COMMA REG COMMA REG {
	     fun l -> let ex = fresh_label() in
		      { label = ex;
	    op = $1;
	    dest = Some (from_string $2);
	    args = [Reg (from_string $4); Reg (from_string $6)];
	    exits = [ex]
	   }, ex
	}
  | ECALL {
	  fun l -> let ex = fresh_label() in
		   { label = ex;
	  op = "ecall";
	    dest = None;
	    args = [];
	    exits = [ex]
	  }, ex
	 }

